import pytest
from workspaces.capability.helpers import CapabilitySequence

from workspaces.capability.schema import CapabilityExecution
from workspaces.capability.services.capability_engine import CapabilityEngine
from workspaces.workflow.schema import WorkflowRequest
from workspaces.workflow.services.workflow_service import WorkflowService

pytest_plugins = ["testing.utils.conftest"]

"""
Tests for CapabilityEngine
"""


@pytest.mark.usefixtures(
    "mock_capability_execution", "mock_capability_engine", "mock_workflow_service"
)
class TestCapabilityEngine:
    @pytest.mark.filterwarnings("ignore::sqlalchemy.exc.SAWarning")
    def test_load_engine(
        self,
        mock_capability_engine: CapabilityEngine,
        mock_capability_execution: CapabilityExecution,
    ):
        engine = mock_capability_engine
        engine.load_engine(mock_capability_execution)

        assert engine.execution == mock_capability_execution

    @pytest.mark.filterwarnings("ignore::sqlalchemy.exc.SAWarning")
    def test_execute(self, mock_capability_engine: CapabilityEngine):
        assert (
            mock_capability_engine.execution.steps
            == "prepare-and-run-workflow null\nawait-workflow"
        )
        assert mock_capability_engine.execution.current_step == 0
        steps = mock_capability_engine.execution.steps
        step = CapabilitySequence.from_str(steps)[0]
        assert step.step_value == "null"
        # Execute is confirmed as getting called with the correct steps at this point.
        # TODO: The following line can't be run without mocked steps, but this method also belongs to steps so...
        # assert step.execute.call_count == 1

    @pytest.mark.filterwarnings("ignore::sqlalchemy.exc.SAWarning")
    def test_submit_workflow_request(
        self,
        mock_capability_execution: CapabilityExecution,
        mock_workflow_service: WorkflowService,
        mock_workflow_requests: WorkflowRequest,
    ):
        # assert mock_workflow_service.create_workflow_request(workflow="null", argument="-g") == mock_workflow_requests[0]
        pass
